import {
  WebSocketGateway,
  SubscribeMessage,
  MessageBody,
  ConnectedSocket,
  OnGatewayConnection,
  OnGatewayDisconnect,
} from '@nestjs/websockets';
import { Socket } from 'socket.io';

@WebSocketGateway({
  cors: {
    origin: '*',
    methods: ['GET', 'HEAD', 'PUT', 'PATCH', 'POST', 'DELETE', 'OPTIONS'],
    credentials: true,
  },
})
export class AppGateway implements OnGatewayConnection, OnGatewayDisconnect {
  private activeUsers: Record<string, string> = {};

  handleConnection(client: Socket) {
    console.log(`Client connected: ${client.id}`);
    this.activeUsers[client.id] = `User-${Object.keys(this.activeUsers).length + 1}`;
    client.broadcast.emit('response', { message: `${this.activeUsers[client.id]} has joined the chat!` });
  }

  handleDisconnect(client: Socket) {
    console.log(`Client disconnected: ${client.id}`);
    const username = this.activeUsers[client.id];
    delete this.activeUsers[client.id];
    client.broadcast.emit('response', { message: `${username} has left the chat.` });
  }

  @SubscribeMessage('message')
  handleMessage(@MessageBody() body: { text: string }, @ConnectedSocket() client: Socket) {
    const username = this.activeUsers[client.id];
    console.log(`${username} says: ${body.text}`);
    client.broadcast.emit('response', { message: `${username}: ${body.text}` });
  }
}
